<?php

namespace Everyware\Plugin\PresentationPreview;

use Everyware\Plugin\PresentationPreview\Contracts\PreviewRenderer;
use Everyware\Plugin\PresentationPreview\Exceptions\UnsupportedContentTypeException;
use Everyware\Plugin\PresentationPreview\Exceptions\UnsupportedMethodException;

class Router
{
    public const PREVIEW_ARTICLE = 'preview-api/v1/article';

    private Request $request;

    private static $validMethods = ['GET', 'HEAD', 'POST', 'OPTIONS'];

    public function __construct(Request $request)
    {
        $this->request = $request;
    }

    public function isApiRequest(): bool
    {
        return $this->request->getPath() === static::PREVIEW_ARTICLE;
    }

    /**
     * @throws UnsupportedMethodException
     * @throws UnsupportedContentTypeException
     */
    public function validateRequest(): void
    {
        $method = $this->request->getMethod();
        if ( ! in_array($method, static::$validMethods)) {
            throw new UnsupportedMethodException(
                "Unsupported method: `$method`. Use `POST` instead."
            );
        }

        if ($this->expectsContent() && ! $this->request->hasContentType('application/json')) {
            throw new UnsupportedContentTypeException(
                "Unsupported content type. Use Content-Type: `application/json` instead."
            );
        }
    }

    public function renderPreview(PreviewRenderer $renderer): void
    {
        if ($this->request->getMethod() === 'POST') {

            $renderer->render(new ArticlePreview($this->request->getBody()));
        }
    }

    /**
     * @throws UnsupportedMethodException
     * @throws UnsupportedContentTypeException
     */
    public function setupAPI(): void
    {
        $this->setCORSHeaders();
        $this->validateRequest();
    }

    public function setCORSHeaders(): void
    {
        // Allow from any origin
        if (isset($_SERVER['HTTP_ORIGIN'])) {
            header('Access-Control-Allow-Origin: *');
        }

        // Access-Control headers are received during OPTIONS "preflight" requests
        if ($this->request->getMethod() === 'OPTIONS') {
            header('Access-Control-Allow-Methods: ' . implode(', ', static::$validMethods));
            header("Access-Control-Allow-Headers: ". implode(', ', [ 'Content-Type', 'Origin']));
        }
    }

    public function expectsContent(): bool
    {
        return ! in_array($this->request->getMethod(), ['HEAD', 'OPTIONS']);
    }
}
