<?php

namespace Spec\Everyware\Plugin\PresentationPreview;

use Everyware\Plugin\PresentationPreview\Contracts\PreviewData;
use Everyware\Plugin\PresentationPreview\ArticlePreview;
use Everyware\Plugin\PresentationPreview\Exceptions\InvalidDataException;
use PhpSpec\ObjectBehavior;

class ArticlePreviewSpec extends ObjectBehavior
{
    private $data = [
        'uuid' => 'article uuid here!',
        'meta' => [
            'imengine' => 'https://yes.hello',
            'concepts' => [
                '13e5896f-e9c7-4481-8bb4-bd48bb69943d',
                '90a0891b-2804-4c89-9515-d042c2f23e18'
            ],
            'articles' => [
                '13e5896f-e9c7-4481-8bb4-bd48bb69943d',
                '90a0891b-2804-4c89-9515-d042c2f23e18'
            ]
        ],
        'content' => "NewsML idf xml here! Can be found on the property 'BodyRaw' in public OC."
    ];

    public function let(): void
    {
        $this->beConstructedWith($this->generateData($this->data));
    }

    public function it_is_initializable(): void
    {
        $this->shouldHaveType(ArticlePreview::class);
        $this->shouldImplement(PreviewData::class);
    }

    public function it_should_throw_error_on_invalid_json(): void
    {
        $this->shouldThrow(InvalidDataException::class)->during('__construct', [' { content: "invalid json" ']);
    }

    public function it_should_throw_error_on_invalid_data(): void
    {
        // Missing "content"
        $this->shouldThrow(InvalidDataException::class)->during('__construct', [
            json_encode([
                'uuid' => 'uuid',
                'meta' => []
            ])
        ]);

        // Missing "meta"
        $this->shouldThrow(InvalidDataException::class)->during('__construct', [
            json_encode([
                'uuid' => 'uuid',
                'content' => ''
            ])
        ]);

        // Missing "uuid"
        $this->shouldThrow(InvalidDataException::class)->during('__construct', [
            json_encode([
                'meta' => [],
                'content' => ''
            ])
        ]);
    }

    public function it_should_convert_the_json_data_to_associative_arrays(): void
    {
        $this->toArray()->shouldReturn($this->data);
    }

    public function it_should_offer_a_getter_for_the_content(): void
    {
        $this->getContent()->shouldReturn($this->data['content']);
    }

    public function it_should_offer_a_getter_for_the_uuid(): void
    {
        $this->getUuid()->shouldReturn($this->data['uuid']);
    }

    public function it_should_offer_a_getter_for_the_metaData(): void
    {
        $this->getMetaData()->shouldReturn($this->data['meta']);
    }

    public function it_should_offer_a_getter_for_the_imengine_uri(): void
    {
        $this->getImengineUri()->shouldReturn($this->data['meta']['imengine']);
    }

    public function it_should_offer_a_getter_for_the_related_articles(): void
    {
        $this->getRelatedArticles()->shouldReturn($this->data['meta']['articles']);
    }

    public function it_should_offer_a_getter_for_the_related_concepts(): void
    {
        $this->getRelatedConcepts()->shouldReturn($this->data['meta']['concepts']);
    }

    public function it_should_return_empty_array_if_no_related_articles_exists(): void
    {
        $data = $this->generateData([
            'uuid' => 'article uuid here!',
            'meta' => [
                'imengine' => 'https://yes.hello'
            ],
            'content' => "NewsML idf xml here! Can be found on the property 'BodyRaw' in public OC."
        ]);
        $this->beConstructedWith($data);
        $this->getRelatedArticles()->shouldReturn([]);
    }

    public function it_should_return_empty_array_if_no_related_concepts_exists(): void
    {
        $data = $this->generateData([
            'uuid' => 'article uuid here!',
            'meta' => [
                'imengine' => 'https://yes.hello'
            ],
            'content' => "NewsML idf xml here! Can be found on the property 'BodyRaw' in public OC."
        ]);

        $this->beConstructedWith($data);
        $this->getRelatedConcepts()->shouldReturn([]);
    }

    private function generateData(array $data)
    {
        return json_encode(['data' => $data]);
    }

}
