<?php

namespace Spec\Everyware\Plugin\PresentationPreview;

use Everyware\Plugin\PresentationPreview\Exceptions\InvalidDataException;
use Everyware\Plugin\PresentationPreview\Input;
use Everyware\Plugin\PresentationPreview\Request;
use JsonException;
use PhpSpec\ObjectBehavior;

class RequestSpec extends ObjectBehavior
{
    /**
     * @var Input
     */
    private $input;

    /**
     * Store a backup of the external $_SERVER array
     * @var array
     */
    private array $serverBackup;

    public function let(Input $input): void
    {
        $this->input = $input;

        // Store the current state of $_SERVER
        $this->serverBackup = $_SERVER;

        $this->beConstructedWith($this->input);
    }

    public function letGo(): void
    {
        // Restore $_SERVER to its original state
        $_SERVER = $this->serverBackup;
    }

    public function it_is_initializable(): void
    {
        $this->shouldHaveType(Request::class);
    }

    public function it_will_determine_the_content_type_of_the_request(): void
    {
        $this->hasContentType('without_content_type')->shouldBe(false);

        $_SERVER['CONTENT_TYPE'] = 'application/json';

        // With only one of the required Server Parameters
        $this->hasContentType('without_content_type')->shouldBe(false);

        $_SERVER['HTTP_CONTENT_TYPE'] = 'application/json';

        // Match with another "Content Type" with the appropriate values in place
        $this->hasContentType('plain/text')->shouldBe(false);

        $this->hasContentType('application/json')->shouldBe(true);
    }

    public function it_will_determine_the_content_type_is_json(): void
    {
        $_SERVER['CONTENT_TYPE'] = 'application/json';
        $_SERVER['HTTP_CONTENT_TYPE'] = 'application/json';

        $this->isJsonRequest()->shouldBe(true);
    }

    public function it_will_extract_the_request_path(): void
    {
        $_SERVER['REQUEST_URI'] = '/preview-api/v1/article';

        $this->getPath()->shouldReturn('preview-api/v1/article');
    }

    public function it_will_ignore_query_parameters_when_extracting_the_request_path(): void
    {
        $_SERVER['REQUEST_URI'] = '/preview-api/v1/article?something=extra';

        $this->getPath()->shouldReturn('preview-api/v1/article');
    }

    public function it_offers_helper_to_match_request_path(): void
    {
        $_SERVER['REQUEST_URI'] = '/preview-api/v1/article?something=extra';

        $this->isPath('preview-api/v1/article')->shouldBe(true);
    }

    public function it_will_get_request_data_as_raw_input(): void
    {
        $data = json_encode([
            'data' =>
                [
                    'uuid' => 'article uuid here!',
                    'meta' => [
                        'imengine' => 'https://yes.hello',
                        'concepts' => [
                            '13e5896f-e9c7-4481-8bb4-bd48bb69943d',
                            '90a0891b-2804-4c89-9515-d042c2f23e18'
                        ],
                        'articles' => [
                            '13e5896f-e9c7-4481-8bb4-bd48bb69943d',
                            '90a0891b-2804-4c89-9515-d042c2f23e18'
                        ]
                    ],
                    'content' => "NewsML idf xml here! Kan be found on the property 'BodyRaw' in public OC."
                ]
        ], JSON_THROW_ON_ERROR);

        $this->input->get()->willReturn($data);

        $this->getBody()->shouldReturn($data);
    }

    public function it_will_throw_exception_if_request_has_no_data(): void
    {
        $this->input->get()->willReturn('');

        $this->shouldThrow(InvalidDataException::class)->during('getBody');
    }
}
