<?php

namespace Spec\Everyware\Plugin\PresentationPreview;

use Everyware\Plugin\PresentationPreview\Contracts\PreviewData;
use Everyware\Plugin\PresentationPreview\Contracts\PreviewRenderer;
use Everyware\Plugin\PresentationPreview\Exceptions\InvalidDataException;
use Everyware\Plugin\PresentationPreview\Exceptions\UnsupportedContentTypeException;
use Everyware\Plugin\PresentationPreview\Exceptions\UnsupportedMethodException;
use Everyware\Plugin\PresentationPreview\Request;
use Everyware\Plugin\PresentationPreview\Router;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class RouterSpec extends ObjectBehavior
{
    private $request;

    public function let(Request $request): void
    {
        $this->request = $request;

        $this->beConstructedWith($request);
    }

    public function it_is_initializable(): void
    {
        $this->shouldHaveType(Router::class);
    }

    public function it_will_verify_an_API_request(): void
    {
        $this->request->getPath()->willReturn(
            'preview-api/v1/article',
            'not-api-request'
        );

        $this->isApiRequest()->shouldReturn(true);
        $this->isApiRequest()->shouldReturn(false);
    }

    public function it_will_throw_exception_if_request_has_unsupported_method(): void
    {
        $this->request->getPath()->willReturn('preview-api/v1/article');
        $this->request->getMethod()->willReturn('PUT');

        $this->shouldThrow(UnsupportedMethodException::class)->during('validateRequest');
    }

    public function it_will_throw_exception_if_request_has_unsupported_contentType(): void
    {
        $this->request->getPath()->willReturn('preview-api/v1/article');
        $this->request->getMethod()->willReturn('POST');

        $this->request->hasContentType('application/json')->willReturn(false);

        $this->shouldThrow(UnsupportedContentTypeException::class)->during('validateRequest');
    }

    public function it_will_not_check_contentType_for_OPTIONS_requests(): void
    {
        $this->request->getPath()->willReturn('preview-api/v1/article');
        $this->request->getMethod()->willReturn('OPTIONS');

        $this->request->hasContentType('application/json')->shouldNotBeCalled();

        $this->validateRequest();
    }

    public function it_will_not_check_contentType_for_HEAD_requests(): void
    {
        $this->request->getPath()->willReturn('preview-api/v1/article');
        $this->request->getMethod()->willReturn('HEAD');

        $this->request->hasContentType('application/json')->shouldNotBeCalled();

        $this->validateRequest();
    }

    public function it_will_render_PreviewRenderer_with_PreviewDataInstance(PreviewRenderer $renderer)
    {
        $this->request->getMethod()->willReturn('POST');
        $this->request->getBody()->willReturn(json_encode([
            'data' =>
                [
                    'uuid' => 'article uuid here!',
                    'meta' => [
                        'imengine' => 'https://yes.hello',
                        'concepts' => [
                            '13e5896f-e9c7-4481-8bb4-bd48bb69943d',
                            '90a0891b-2804-4c89-9515-d042c2f23e18'
                        ],
                        'articles' => [
                            '13e5896f-e9c7-4481-8bb4-bd48bb69943d',
                            '90a0891b-2804-4c89-9515-d042c2f23e18'
                        ]
                    ],
                    'content' => "NewsML idf xml here! Kan be found on the property 'BodyRaw' in public OC."
                ]
        ], JSON_THROW_ON_ERROR));

        $renderer->render(Argument::type(PreviewData::class))->shouldBeCalled();

        $this->renderPreview($renderer);
    }

    public function it_will_not_render_none_POST_requests(PreviewRenderer $renderer)
    {
        $this->request->getMethod()->willReturn('GET');

        $this->request->getBody()->shouldNotBeCalled();
        $renderer->render()->shouldNotBeCalled();

        $this->renderPreview($renderer);
    }

    public function it_should_throw_exception_if_body_is_invalid_during_preview_rendering(PreviewRenderer $renderer)
    {
        $this->request->getMethod()->willReturn('POST');

        $this->request->getBody()->willReturn('InvalidJson');
        $renderer->render(Argument::type(PreviewData::class))->shouldNotBeCalled();

        $this->shouldThrow(InvalidDataException::class)->during('renderPreview', [$renderer]);
    }

    public function it_can_determine_if_a_request_expects_content()
    {
        $this->request->getMethod()->willReturn('HEAD', 'OPTIONS', 'POST');
        $this->expectsContent()->shouldBe(false);
        $this->expectsContent()->shouldBe(false);
        $this->expectsContent()->shouldBe(true);
    }
}
